package AspectObserverExample;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
/**
 * A Slider that notifies its observers
 */
class ObservedSlider {
  private PropertyChangeSupport listeners= new PropertyChangeSupport(this);
  /**
   * Adds observer notification to a setter method
   */
  hook setValueNotify {
    setValueNotify(method(Object value)) {
      execute(method);
    }
    after() {
      System.out.println("notifying observers of value change to " + value);
      global.firePropertyChange("value", null, value);
    }
  }
  /**
   * Removes a PropertyChangeListener
   */
  public void removePropertyChangeListener(PropertyChangeListener aListener) {
    listeners.removePropertyChangeListener(aListener);
  }
  /**
   * Fires a PropertyChange notification to all listeners
   */
  public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
    listeners.firePropertyChange(propertyName, oldValue, newValue);
  }
  /**
   * Adds a PropertyChangeListener to be notified
   */
  public void addPropertyChangeListener(PropertyChangeListener aListener) {
    listeners.addPropertyChangeListener(aListener);
  }
}