import java.awt.Point;
import java.awt.Color;
import java.util.Enumeration;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Panel;
import java.util.Vector;
import java.awt.Component;
/**
 * 
 */
public class Ball extends Panel implements Bouncer, Runnable {
  private Vector observerBallCollection= new Vector();
  private Color ballColor;
  private int speed;
  private boolean running;
  private int dY;
  private int dX;
  private int ballSize;
  public Ball (int x, int y)
  {
    ballSize = 10;
    ballColor = new Color (255, 255, 0);
    dX = -3;
    dY = -2;
    running = false;
    speed = 20;
    this.setBounds (x - (ballSize / 2), y - ballSize, ballSize, ballSize);
    this.setBackground (ballColor);
    this.setEnabled (false);
    this.setVisible (true);
  }
  /**
   * Moves the ball to its next position.
   */
  public void move() {
    // Move by dx and dy, bouncing off the side of the main window
    {
      java.awt.Point currentLoc = getLocation();
      setLocation (currentLoc.x + dX, currentLoc.y + dY);
    }
  }
  /**
   * 
   */
  public Point getNextLocation() {
    {
      Point currentLoc = getLocation();
      return new Point (currentLoc.x + dX, currentLoc.y + dY);
    }
  }
  /**
   * Let's the ball move and bounce in a thread.
   */
  public void run() {
    
    running = true;
    
    while (running)
    {
      try
      {
        Dimension size = getSize();
        Rectangle windowBounds = getParent().getBounds();
        Point nextLoc = getNextLocation();
    
        // Bounce against top, left or right wall?
        if (nextLoc.x <= windowBounds.x ||
            nextLoc.x + size.width >= windowBounds.x + windowBounds.width)
          bounceHorizontal();
        else if (nextLoc.y <= windowBounds.y)
          bounceVertical();
        else if (nextLoc.y + size.height >= windowBounds.y + windowBounds.height)
        {
          running = false;
          notifyObservers();
        }
        else
        {
          // Bounce against object?
          Component c = getParent().getComponentAt (nextLoc);
          if (c != null)
          {
            if (c instanceof Bounceable)
            {
              Bounceable b = (Bounceable) c;
              b.bounces (this);
            }
          }
        }
       
        move ();
        Thread.sleep (speed);
      }
      catch(InterruptedException e)
      {
        System.out.println("Catch Interrupt Exception");
      }
    }
  }
  /**
   * Bounces the ball in vertical direction.
   */
  public void bounceVertical() {
    {
      java.awt.Toolkit.getDefaultToolkit().beep();
      dY = -dY;
    }
  }
  /**
   * Bounces the ball in horizontal direction.
   */
  public void bounceHorizontal() {
    {
      java.awt.Toolkit.getDefaultToolkit().beep();
      dX = -dX;
    }
  }
  /**
   * Notifies attached observers of a change by invoking update.
   */
  public void notifyObservers() {
    ObserverBall observer;
    Enumeration obs = getObserversBall().elements();
    while (obs.hasMoreElements()) {
      observer = (ObserverBall) obs.nextElement();
      observer.update(this);
    }
  }
  /**
   * Gets the collection of properties
   */
  public Vector getObserversBall() {
    return observerBallCollection;
  }
  /**
   * Adds an element to the collection of properties
   */
  public void addObserver(ObserverBall aObserverBall) {
    observerBallCollection.addElement(aObserverBall);
  }
  /**
   * Removes an element from the collection of properties
   */
  public void deleteObserver(ObserverBall aObserverBall) {
    observerBallCollection.removeElement(aObserverBall);
  }
}