import java.awt.Button;
import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.Frame;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.Toolkit;
import java.awt.Panel;
import java.awt.Label;
import java.awt.event.WindowEvent;
import java.awt.AWTEvent;
import java.awt.event.MouseMotionListener;
/**
 * Breakout game main class.
 */
public class Breakout extends Frame implements ObserverBall, MouseMotionListener {
  private int brickCols;
  private Paddle aPaddle;
  private Ball aBall;
  private int brickRows;
  private Button startBtn;
  private Panel pane;
  private Color backgroundColor;
  private Dimension resolution;
  // Handle click on [x] box
  protected void processWindowEvent (WindowEvent e)
  {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) System.exit(0);
  }
  public static void main (String [] args)
  {
    Breakout b = new Breakout ();
  }
  /**
   * Moves the paddle when dragged.
   */
  public void mouseDragged(MouseEvent aMouseEvent) {
    int width = aPaddle.getSize().width, halfWidth = width / 2;
    int x = aMouseEvent.getX() + aPaddle.getLocation().x - halfWidth;
                              
    if (x < 0)
      x = 0;
    else if (x > pane.getSize().width - width)
      x = pane.getSize().width - width;
    aPaddle.move (x);
  }
  /**
   * Receives and handles a GameOver event.
   */
  public void update(Ball aBall) {
    gameOver();
  }
  /**
   * Ignored.
   */
  public void mouseMoved(MouseEvent aMouseEvent) {
    
  }
  /**
   * 
   */
  public Breakout() {
    // From static initializers
    pane = new Panel();
    brickRows = 4;
    brickCols = 5;
    resolution = new Dimension (240, 320);
    backgroundColor = new Color (128, 64, 255);
    
    // From constructor method
    add (pane);
    pane.setLayout (null);
    setTitle ("Breakout (" + resolution.width + "x" + resolution.height + ")");
    pane.setBackground (backgroundColor);
    setBounds (0, 0, resolution.width, resolution.height);
    setResizable (false);
    setVisible(true);
    reset ();
  }
  /**
   * Starts the game.
   */
  public void startBtnClicked(ActionEvent aActionEvent) {
    pane.remove (startBtn);
    Thread thread = new Thread(aBall);
    //<Running> = true;
    thread.start();
  }
  /**
   * Displays a Game Over message and resets the game.
   */
  public void gameOver() {
    
    try
    {
      Label label = new Label("Game Over");
      label.setBounds(pane.getSize().width / 2 - 50,  pane.getSize().height / 2 - 10, 100, 20);
      pane.add(label);
      Toolkit.getDefaultToolkit().beep();
      Thread.sleep(400);
      Toolkit.getDefaultToolkit().beep();
      Thread.sleep(200);
      Toolkit.getDefaultToolkit().beep();
      Thread.sleep(200);
      Toolkit.getDefaultToolkit().beep();
      Thread.sleep(400);
      Toolkit.getDefaultToolkit().beep();
      Thread.sleep(800);
      Toolkit.getDefaultToolkit().beep();
      Thread.sleep(400);
      Toolkit.getDefaultToolkit().beep();
      reset ();
    }
    catch (Exception e)
    {
      System.out.println ("Exception: " + e.toString());
    }
  }
  /**
   * (Re-)initialises the game.
   */
  public void reset() {
    
    pane.removeAll();
    if (aBall != null)
    {
      aBall.deleteObserver(this);
    }
    
    // Add the bricks
    Dimension size = pane.getSize();
    Dimension brickSize = new Dimension (size.width / brickCols, size.height / (3 * brickRows));
    for (int i = 0; i < brickRows; i++)
      for (int j = 0; j < brickCols; j++)
      {
        Brick brick = new Brick (j * brickSize.width, i * brickSize.height, brickSize);
        pane.add (brick);
      }
    
    // Add the paddle
    Dimension paddleSize = new Dimension (size.width / 4, brickSize.height);
    aPaddle = new Paddle (size.width / 2, size.height, paddleSize);
    aPaddle.addMouseMotionListener(this);
    pane.add (aPaddle);
    
    // Add the ball
    aBall = new Ball (size.width / 2, aPaddle.getLocation().y - 10);
    aBall.addObserver(this);
    pane.add (aBall);
    
    // Add start button
    startBtn = new Button("Start");
    startBtn.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        startBtnClicked(e);
      }
    });
    startBtn.setBounds(getSize().width / 2 - 30, getSize().height / 2 - 10, 60, 20);
    pane.add (startBtn);
    
    // Start playing
    doLayout();
    enableEvents (AWTEvent.WINDOW_EVENT_MASK);
  }
}