/**
 * Event data class for SEESCOA.
 EventData stores a date/time in a precision of milliseconds, a
 component ID and a comment.
@author Miro Casanova
@author Dennis Wagelaar
@version 15 October 2002
 */
public class EventData extends TimeData {
  private ID componentID= null;
  private String comment= null;
  /**
   * Constructs an EventData.
   * @author Miro Casanova
   * @author Dennis Wagelaar
   */
  public EventData() {
    super();
  } //EventData
  /**
   * Constructs an EventData.
   * @param time The time data
   * @param compid The component ID
   * @param comment The event comment
   * @author Dennis Wagelaar
   */
  public EventData(TimeData time, ID compid, String comment) {
    super(time);
    this.setComment(comment);
    this.setComponentID(compid);
  } //EventData
  /**
   * Constructs an EventData.
   * @param year The year (e.g. 2002)
   * @param month The month (e.g. 12)
   * @param day The day of the month (e.g. 31)
   * @param hour The hour (e.g. 23)
   * @param minute The minute (e.g. 59)
   * @param second The second (e.g. 59)
   * @param msecond The amount of milliseconds (e.g. 999)
   * @param compid The component ID
   * @param comment The event comment
   * @author Miro Casanova
   * @author Dennis Wagelaar
   */
  public EventData(int year, int month, int day, int hour, int minute, int second, int msecond, ID compid, String comment) {
    super(year, month, day, hour, minute, second, msecond);
    this.setComment(comment);
    this.setComponentID(compid);
  } //EventData
  /**
   * Gets the property value
   */
  public String getComment() {
    return this.comment;
  }
  /**
   * Sets the property value
   */
  public void setComment(String comment) {
    this.comment = comment;
  }
  /**
   * Gets the property value
   */
  public ID getComponentID() {
    return this.componentID;
  }
  /**
   * Sets the property value
   */
  public void setComponentID(ID componentID) {
    this.componentID = componentID;
  }
  /**
   * Generates an ID from the ComponentID
   */
  public ID getID() {
    return new ID(this.getComponentID().getIDstring() + "-" + this.getTimestamp() + "-" + this.getMilliSec());
  }
}