/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package implementations;

import java.util.*;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.util.*;

/**
 * Implementation generator plugin class for implementing a SingleRelation
 * composite in the SEESCOA component language.
 *
 * @author Dennis Wagelaar
 * @version 0.1.18, 22 November 2003
 */

public class SEESCOASingleRelation extends JavaSingleRelation
{
  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Applies the implementation generator to an Owner concept.
   *
   * @param owner The concept to apply the role part for.
   * @param properties The set of property concepts.
   * @throws CoComposeException if generating implementation failed.
   */
  protected void applyToOwner(Concept owner, Collection properties)
    throws CoComposeException
  {
    Composite composite = getComposite();
    Collection variables = new Vector();
    Iterator propertyIt;
    Concept property;
    String impl, form;

    impl = getCurrentImplementation(owner);
    // generate methods
    propertyIt = properties.iterator();
    while (propertyIt.hasNext())
    {
      property = (Concept) propertyIt.next();
      form = property.getForm();
      if (form.endsWith("Port"))
      {
        // add port header
        variables.add("public " + form + " " + property.getName() + ";");
      }
      else
      {
        // add pointer to property, since property implementation may not be
        // finished.
        impl = insertMethod(impl, "<" + property.getId() + ">");
      }
    }
    // insert variables
    if (!variables.isEmpty())
      impl = insertVariables(impl, variables);
    setCurrentImplementation(owner, impl);
  } //applyToOwner
} //class SEESCOASingleRelation